function PadePoly(fx,xRange,Lb,Ub,runNum,sFxName,diaryFilename)
%SHAMPOLY2 Summary of this function goes here
%   Detailed explanation goes here
clc
global bDeleteIfExists
global bUseDiary
global xdata
global ydata
global orderA
global orderB
warning('off','all')
if isempty(sFxName)
  sFxName = getFuncName(fx);
end
xdata = xRange';
ydata = xdata;
for i=1:length(xdata)
  ydata(i)=fx(xdata(i));
end



xmin = min(xdata);
xmax = max(xdata);
ymin = min(ydata);
ymax = max(ydata);
xdata = (xdata - xmin)/(xmax - xmin) + 1;
ydata = (ydata - ymin)/(ymax - ymin) + 1;
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);

options = optimoptions('particleswarm', 'Display', 'iter');
[x,psAICc] = particleswarm(@optimFunc,2,Lb,Ub,options);
orderA = round(x(1));
orderB = round(x(2));
if bUseDiary
  diaryFilename = strrep(diaryFilename, ".txt", strcat("_", num2str(orderA),"x",num2str(orderB),"_run", num2str(runNum),".txt"));
  if exist(diaryFilename, 'file')==2
    if bDeleteIfExists
      delete(diaryFilename);
    else
      return;
    end
  end
end
X = [];
for i=1:orderA
  xs = xdata.^i;
  X = [X;xs'];
end
for i=1:orderB
  xs = ydata.*xdata.^i;
  X = [X;xs'];
end
X = X';
lm = fitlm(X,ydata);
if bUseDiary
  diary(diaryFilename)
end
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);

format long
disp(lm);
anva = anova(lm,'summary');
disp(anva);
format short
fprintf('orderA = %f, orderB = %f\n', orderA, orderB);
fprintf("Xmin = %f and Xmax = %f\n", xmin, xmax);
fprintf("Ymin = %f and Ymax = %f\n", ymin, ymax);
sMdl1 = "y = [Intercept";
j = 1;
for i=1:orderA
  if i<2
    sMdl1 = strcat(sMdl1, " + cx", num2str(j),"*x");
  else
    sMdl1 = strcat(sMdl1, " + cx", num2str(j),"*x^",num2str(i));
  end
  j = j + 1;
end
sMdl1 = strcat(sMdl1, "] /");
sMdl2 = "[ 1";
for i=1:orderB
  if i<2
    sMdl2 = strcat(sMdl2, " + cx", num2str(j),"*y*x");
  else
    sMdl2 = strcat(sMdl2, " + cx", num2str(j),"*y*x^",num2str(i));
  end
  j = j + 1;
end
sMdl2 = strcat(sMdl2, "]");
fprintf("Model is %s\n\t\t%s\n", sMdl1, sMdl2);
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);
n = length(xdata);
sumsqr = 0;
for i=1:n
  yc = lm.Coefficients{1,1};
  for j=1:orderA
    yc = yc + lm.Coefficients{j+1,1} * xdata(i)^j;
  end
  for j=1:orderB
    yc = yc + lm.Coefficients{j+orderA+1,1} * ydata(i)*xdata(i)^j;
  end
  sumsqr = sumsqr + (ydata(i) - yc)^2;
end

k = orderA + orderB + 1;
fprintf('MSS of errors squared = %e\n', sqrt(sumsqr)/n);
AIC = lm.ModelCriterion.AIC;
AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
fprintf('Particle swarm AICc =  %e\n', psAICc);
fprintf('AIC =  %e\n', AIC);
fprintf('AICc =  %e\n', AICc);

if bUseDiary
  diary off
end
end

function AICc = optimFunc(x)
  global xdata
  global ydata
  global orderA
  global orderB
  
  orderA = round(x(1));
  orderB = round(x(2));
  X = [];
  for i=1:orderA
    xs = xdata.^i;
    X = [X;xs'];
  end
  for i=1:orderB
    xs = ydata.*xdata.^i;
    X = [X;xs'];
  end
  X = X';
  lm = fitlm(X,ydata);

  n = length(xdata);
  k = orderA + orderB + 1;
  AIC = lm.ModelCriterion.AIC;
  AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
  if isinf(AICc), AICc = -1e+99; end
end

function sFx = getFuncName(fx)
  sFx = func2str(fx);
  if sFx(1:2)=="@("
    i = strfind(sFx,")");
    sFx = sFx(i(1)+1:end);
  elseif sFx(1)=="@"
    sFx = strcat(sFx(2:end),".m");
  else
    % return sFx as is
  end
end